@@ -9,7 +9,7 @@
 
 package B::C;
 
-our $VERSION = '1.04_26';
+our $VERSION = '1.04_27';
 
 package B::C::Section;
 
@@ -552,12 +552,12 @@ package B::FAKEOP;
 
 our @ISA = qw(B::OP);
 
-sub B::FAKEOP::new {
+sub new {
   my ( $class, %objdata ) = @_;
   bless \%objdata, $class;
 }
 
-sub B::FAKEOP::save {
+sub save {
   my ( $op, $level ) = @_;
   $opsect->add(
     sprintf( "%s, %s, %s", $op->next, $op->sibling, $op->_save_common_middle )
@@ -568,13 +568,14 @@ sub B::FAKEOP::save {
   return "&op_list[$ix]";
 }
 
-sub B::FAKEOP::next    { $_[0]->{"next"}  || 0 }
-sub B::FAKEOP::type    { $_[0]->{type}    || 0 }
-sub B::FAKEOP::sibling { $_[0]->{sibling} || 0 }
-sub B::FAKEOP::ppaddr  { $_[0]->{ppaddr}  || 0 }
-sub B::FAKEOP::targ    { $_[0]->{targ}    || 0 }
-sub B::FAKEOP::flags   { $_[0]->{flags}   || 0 }
-sub B::FAKEOP::private { $_[0]->{private} || 0 }
+*_save_common_middle = \&B::OP::_save_common_middle;
+sub next    { $_[0]->{"next"}  || 0 }
+sub type    { $_[0]->{type}    || 0 }
+sub sibling { $_[0]->{sibling} || 0 }
+sub ppaddr  { $_[0]->{ppaddr}  || 0 }
+sub targ    { $_[0]->{targ}    || 0 }
+sub flags   { $_[0]->{flags}   || 0 }
+sub private { $_[0]->{private} || 0 }
 
 package B::C;
 
